"""Utility functions for 0G Storage SDK."""

try:
    from .error_handler import (
        ErrorContext,
        wrap_with_context,
        handle_network_error,
        handle_timeout_error,
        handle_upload_error,
        handle_download_error,
        handle_node_error,
        handle_transaction_error,
        validate_input,
        is_retryable,
        log_error_details,
        safe_call
    )
except ImportError:
    # Error handler utilities not available, but this is optional
    pass

try:
    from .transfer import (
        RetryOpts,
        get_shard_configs,
        calculate_price,
        delay,
        get_split_num,
        segment_range,
        wait_for_receipt,
        tx_with_gas_adjustment,
        submit_with_gas_adjustment
    )
except ImportError:
    # Transfer utilities not available - may be due to missing dependencies
    pass
